@@ -53,6 +53,23 @@ impl fmt::Display for Method {
     }
 }
 
+#[derive(Clone, Copy, PartialEq, Debug)]
+pub(crate) enum Port {
+    ImplicitHttp,
+    ImplicitHttps,
+    Explicit(u32),
+}
+
+impl Port {
+    pub(crate) fn port(self) -> u32 {
+        match self {
+            Port::ImplicitHttp => 80,
+            Port::ImplicitHttps => 443,
+            Port::Explicit(port) => port,
+        }
+    }
+}
+
 /// An HTTP request.
 ///
 /// Generally created by the [`minreq::get`](fn.get.html)-style
@@ -72,7 +89,7 @@ impl fmt::Display for Method {
 pub struct Request {
     pub(crate) method: Method,
     pub(crate) host: URL,
-    pub(crate) port: u32,
+    pub(crate) port: Port,
     resource: URL,
     headers: HashMap<String, String>,
     body: Option<Vec<u8>>,
@@ -132,7 +149,7 @@ impl Request {
         // Checks if the resource already has a query parameter
         // mentioned in url and if true, adds '&' to add one more
         // parameter or adds '?' to add the first parameter
-        if self.resource.contains("?") {
+        if self.resource.contains('?') {
             self.resource.push('&');
         } else {
             self.resource.push('?');
@@ -296,11 +313,17 @@ impl Request {
 
     fn get_http_head(&self) -> String {
         let mut http = String::with_capacity(32);
+
         // Add the request line and the "Host" header
         http += &format!(
-            "{} {} HTTP/1.1\r\nHost: {}\r\n",
+            "{} {} HTTP/1.1\r\nHost: {}",
             self.method, self.resource, self.host
         );
+        if let Port::Explicit(port) = self.port {
+            http += &format!(":{}", port);
+        }
+        http += "\r\n";
+
         // Add other headers
         for (k, v) in &self.headers {
             http += &format!("{}: {}\r\n", k, v);
@@ -308,10 +331,11 @@ impl Request {
 
         if self.method == Method::Post || self.method == Method::Put || self.method == Method::Patch
         {
-            if let None = self.headers.keys().find(|key| {
+            let not_length = |key: &String| {
                 let key = key.to_lowercase();
-                key == "content-length" || key == "transfer-encoding"
-            }) {
+                key != "content-length" && key != "transfer-encoding"
+            };
+            if self.headers.keys().all(not_length) {
                 // A user agent SHOULD send a Content-Length in a request message when no Transfer-Encoding
                 // is sent and the request method defines a meaning for an enclosed payload body.
                 // refer: https://tools.ietf.org/html/rfc7230#section-3.3.2
@@ -396,7 +420,7 @@ impl Request {
     }
 }
 
-fn parse_url(url: URL) -> (bool, URL, u32, URL) {
+fn parse_url(url: URL) -> (bool, URL, Port, URL) {
     enum UrlParseStatus {
         Protocol,
         AtFirstSlash,
@@ -445,7 +469,13 @@ fn parse_url(url: URL) -> (bool, URL, u32, URL) {
     }
     // Set appropriate port
     let https = url.starts_with("https://");
-    let port = u32::from_str_radix(&port, 10).unwrap_or_else(|_| if https { 443 } else { 80 });
+    let port = port.parse::<u32>().map(Port::Explicit).unwrap_or_else(|_| {
+        if https {
+            Port::ImplicitHttps
+        } else {
+            Port::ImplicitHttp
+        }
+    });
     (https, host, port, resource)
 }
 
