@@ -1,8 +1,10 @@
 use crate::{connection::HttpStream, Error};
 use std::collections::HashMap;
-use std::io::{Bytes, Read};
+use std::io::{Bytes, ErrorKind, Read};
 use std::str;
 
+const MAX_CONTENT_LENGTH: usize = 16 * 1024;
+
 /// An HTTP response.
 ///
 /// Returned by [`Request::send`](struct.Request.html#method.send).
@@ -33,9 +35,17 @@ impl Response {
         let mut body = Vec::new();
         if !is_head && parent.status_code != 204 && parent.status_code != 304 {
             for byte in &mut parent {
-                let (byte, length) = byte?;
-                body.reserve(length);
-                body.push(byte);
+                match byte {
+                    Ok((byte, length)) => {
+                        body.reserve(length);
+                        body.push(byte);
+                    }
+                    Err(Error::IoError(err)) if err.kind() == ErrorKind::WouldBlock => {
+                        // Busy waiting isn't ideal, but waiting for N milliseconds would be worse.
+                        std::thread::yield_now();
+                    }
+                    Err(err) => return Err(err),
+                }
             }
         }
 
@@ -181,7 +191,9 @@ impl Response {
 /// the future (including this byte). Note, however, that the `usize`
 /// can change, particularly when the `Transfer-Encoding` is
 /// `chunked`: then it will reflect how many bytes are left of the
-/// current chunk.
+/// current chunk. The expected size is capped at 16 KiB to avoid
+/// server-side DoS attacks targeted at clients accidentally reserving
+/// too much memory.
 ///
 /// # Example
 /// ```no_run
@@ -281,7 +293,8 @@ fn read_with_content_length(
 
         if let Some(byte) = bytes.next() {
             match byte {
-                Ok(byte) => return Some(Ok((byte, *content_length + 1))),
+                // Cap Content-Length to 16KiB, to avoid out-of-memory issues.
+                Ok(byte) => return Some(Ok((byte, (*content_length).min(MAX_CONTENT_LENGTH) + 1))),
                 Err(err) => return Some(Err(Error::IoError(err))),
             }
         }
@@ -337,7 +350,7 @@ fn read_chunked(
         let incoming_length = if length_line.is_empty() {
             0
         } else {
-            let length = if let Some(i) = length_line.find(";") {
+            let length = if let Some(i) = length_line.find(';') {
                 length_line[..i].trim()
             } else {
                 length_line.trim()
@@ -380,7 +393,7 @@ fn read_chunked(
                         }
                     }
 
-                    return Some(Ok((byte, *chunk_length + 1)));
+                    return Some(Ok((byte, (*chunk_length).min(MAX_CONTENT_LENGTH) + 1)));
                 }
                 Err(err) => return Some(Err(Error::IoError(err))),
             }
@@ -483,19 +496,27 @@ fn read_line(
 ) -> Result<String, Error> {
     let mut bytes = Vec::with_capacity(32);
     for byte in stream {
-        let byte = byte?;
-        if let Some(max_len) = max_len {
-            if bytes.len() >= max_len {
-                return Err(overflow_error);
+        match byte {
+            Ok(byte) => {
+                if let Some(max_len) = max_len {
+                    if bytes.len() >= max_len {
+                        return Err(overflow_error);
+                    }
+                }
+                if byte == b'\n' {
+                    if let Some(b'\r') = bytes.last() {
+                        bytes.pop();
+                    }
+                    break;
+                } else {
+                    bytes.push(byte);
+                }
             }
-        }
-        if byte == b'\n' {
-            if let Some(b'\r') = bytes.last() {
-                bytes.pop();
+            Err(err) if err.kind() == ErrorKind::WouldBlock => {
+                // Busy waiting isn't ideal, but waiting for N milliseconds would be worse.
+                std::thread::yield_now();
             }
-            break;
-        } else {
-            bytes.push(byte);
+            Err(err) => return Err(Error::IoError(err)),
         }
     }
     String::from_utf8(bytes).map_err(|_error| Error::InvalidUtf8InResponse)
@@ -522,9 +543,7 @@ fn parse_status_line(line: &str) -> (i32, String) {
     }
 
     if let Ok(status_code) = status_code.parse::<i32>() {
-        if !reason_phrase.is_empty() {
-            return (status_code, reason_phrase);
-        }
+        return (status_code, reason_phrase);
     }
 
     (503, "Server did not provide a status line".to_string())
